@@ -1,7 +1,7 @@
 from .keyfollower import KeyFollower
 import logging
 import numpy as np
-from .utils import get_position, create_dataset, append_data
+from .utils import get_position, create_dataset, append_data, refresh_dataset
 import sys
 from time import sleep
 
@@ -20,32 +20,23 @@ class DataSource:
     Parameters
     ----------
 
-    h5file: h5py.File
-        Instance of h5py.File object. Choose the file containing data you wish
-        to follow.
+    key_datasets: list
+        A list of key datasets from the hdf5 file.
 
-    keypaths: list
-        A list of paths (as strings) to key datasets in the hdf5 file. Can also be
-        the path to groups, but the groups must contain only key datasets.
-
-    dataset_paths: list
-        A list of paths (as strings) to datasets in h5file that you wish
-        to return frames from (Note: paths must be to the dataset and not
-                               to the group containing it).
+    datasets: dict
+        A dictionary of paths (as strings) to datasets that you wish
+        to return frames from.
 
     timeout: int (optional)
         The maximum time allowed for a dataset to update before the timeout
         termination condition is triggered and iteration is halted. If a value
         is not set this will default to 10 seconds.
 
-    finished_dataset: string (optional)
+    finished_dataset: dataset (optional)
         Path to a scalar hdf5 dataset which is zero when the file is being
         written to and non-zero when the file is complete. Used to stop
         the iterator without waiting for the timeout.
 
-    cache_dataset: bool (optional)
-        Hold a reference to the dataset. Can lead to an increase in read performance,
-        but can cause issues with SWMR and VDS.
 
     use_direct_chunk: bool (optional)
         If dataset chunking is aligned to a single frame, and data is blosc
@@ -53,7 +44,7 @@ class DataSource:
         for performance.
 
     interleaved_paths: dict (optional)
-        A dictionary of string to lists of dataset paths. Where frames are written by multiple file
+        A dictionary of string to lists of datasets. Where frames are written by multiple file
         writers in an interleaved fashion, this will take frames alternating from each
         dataset in the list. Key dataset should be a VDS of interleaved keys. For use where
         direct chunk read is required but not possible through a VDS stack of frames. Only
@@ -64,7 +55,9 @@ class DataSource:
     --------
 
     >>> with h5py.File("/home/documents/work/data/example.h5", "r", swmr = True) as f:
-    >>>     df = DataSource(f, path_to_key_group, path_to_datasets)
+    >>>     keys = [f["key"]]
+    >>>     data = {"data" : f["data"]}
+    >>>     df = DataSource([keys], data, path_to_datasets)
     >>>     for frame_dict in df:
     >>>         print(frame_dict)
 
@@ -72,55 +65,49 @@ class DataSource:
 
     def __init__(
         self,
-        h5file,
-        keypaths,
-        dataset_paths,
+        key_datasets,
+        datasets,
         timeout=10,
         finished_dataset=None,
-        cache_datasets=False,
         use_direct_chunk=False,
-        interleaved_paths=None,
+        interleaved_datasets=None,
     ):
-        self.h5file = h5file
-        self.dataset_paths = dataset_paths
-        self.interleaved_paths = interleaved_paths
+
+        self._datasets = datasets
+        self._interleaved_datasets = interleaved_datasets
         self.max_index = -1
-        self.cache = {}
-        self.cache_datasets = cache_datasets
-        self.interleaved_cache = {}
-        self.kf = KeyFollower(h5file, keypaths, timeout, finished_dataset)
+        self.frame_readers = {}
+        self.interleaved_frame_readers = {}
+        self.kf = KeyFollower(key_datasets, timeout, finished_dataset)
         self.kf.check_datasets()
 
-        if dataset_paths is None and interleaved_paths is None:
+        if datasets is None and interleaved_datasets is None:
             raise RuntimeError("No data specified to follow!")
 
         self._add_datasets_to_cache(use_direct_chunk)
         self._add_interleaved_datasets_to_cache(use_direct_chunk)
 
     def _add_datasets_to_cache(self, use_direct_chunk):
-        if self.dataset_paths is not None:
-            for path in self.dataset_paths:
-                self.cache[path] = FrameReader(
-                    path,
-                    self.h5file,
+        if self._datasets is not None:
+            for path, data in self._datasets.items():
+                self.frame_readers[path] = FrameReader(
+                    data,
                     self.kf.scan_rank,
                     use_direct_chunk=use_direct_chunk,
                 )
 
     def _add_interleaved_datasets_to_cache(self, use_direct_chunk):
-        if self.interleaved_paths is not None:
-            for path, path_list in self.interleaved_paths.items():
-                self.interleaved_cache[path] = []
-                for ds_path in path_list:
+        if self._interleaved_datasets is not None:
+            for path, data_list in self._interleaved_datasets.items():
+                self.interleaved_frame_readers[path] = []
+                for data in data_list:
                     fr = FrameReader(
-                        ds_path,
-                        self.h5file,
+                        data,
                         self.kf.scan_rank,
                         use_direct_chunk=use_direct_chunk,
-                        cache_datasets=self.cache_datasets,
                     )
 
-                    self.interleaved_cache[path].append(fr)
+                    self.interleaved_frame_readers[path].append(fr)
 
     def __iter__(self):
         return self
@@ -129,7 +116,7 @@ class DataSource:
 
         current_dataset_index = next(self.kf)
         force_refresh = False
-        if self.max_index < self.kf.current_max:
+        if self.max_index < current_dataset_index:
             self.max_index = self.kf.current_max
             force_refresh = True
 
@@ -143,19 +130,11 @@ class DataSource:
         return output
 
     def _add_datasets_to_output(self, current_dataset_index, output, force_refresh):
-        if self.dataset_paths is None:
+        if self._datasets is None:
             return
 
-        for path in self.dataset_paths:
-            fg = self.cache.get(
-                path,
-                FrameReader(
-                    path,
-                    self.h5file,
-                    self.kf.scan_rank,
-                    cache_datasets=self.cache_datasets,
-                ),
-            )
+        for path in self._datasets.keys():
+            fg = self.frame_readers[path]
             frame, slice_metadata = fg.read_frame(
                 current_dataset_index, force_refresh=force_refresh
             )
@@ -168,10 +147,10 @@ class DataSource:
     def _add_interleaved_datasets_to_output(
         self, current_dataset_index, output, force_refresh
     ):
-        if self.interleaved_paths is None:
+        if self._interleaved_datasets is None:
             return
 
-        for path, frs in self.interleaved_cache.items():
+        for path, frs in self.interleaved_frame_readers.items():
             n_frs = len(frs)
             fr_index = current_dataset_index % (n_frs)
 
@@ -202,7 +181,7 @@ class DataSource:
         return append_data(data, slice_metadata, dataset)
 
     def is_scan_finished(self):
-        return self.kf._finish_tag
+        return self.kf.finished_set
 
     def has_timed_out(self):
         return self.kf.timed_out
@@ -224,12 +203,8 @@ class FrameReader:
 
     Parameters
     ----------
-    dataset : str
-        The full path to the dataset to extract frames from in the hdf5_File
-
-    h5file : h5py.File
-        Instance of h5py.File object. Choose the file containing dataset you
-        want to extract frames from.
+    dataset : dataset
+        The dataset to extract frames from in the hdf5_File
 
     scan_rank: int
         The rank of the "non-data-frame" part of the N-dimensional dataset
@@ -243,38 +218,28 @@ class FrameReader:
         compressed, will use direct chunk read and compression outside of h5py
         for performance.
 
-    cache_datasets: bool (optional)
-        Hold a reference to the dataset. Can lead to an increase in read performance,
-        but can cause issues with SWMR and VDS.
-
     Examples
     --------
 
     >>> #open and hdf5 file using context manager
     >>> with h5py.File("/path/to/file") as f:
-    >>>     fr = FrameReader(f, "/path/to/dataset", 2)
+    >>>     data = f["data"]
+    >>>     fr = FrameReader(data, 2)
     >>>     #call methods on fg to get the data that you want
 
     """
 
-    def __init__(
-        self, dataset, h5file, scan_rank, use_direct_chunk=False, cache_datasets=False
-    ):
+    def __init__(self, dataset, scan_rank, use_direct_chunk=False):
         self.dataset = dataset
-        self.h5file = h5file
         self.scan_rank = scan_rank
         self.use_direct_chunk = use_direct_chunk
-        self.ds = None
-        if cache_datasets:
-            self.ds = self.h5file[self.dataset]
 
         if use_direct_chunk:
-            ds = self.ds if self.ds is not None else self.h5file[self.dataset]
             self.use_direct_chunk = False
-            prop_dcid = ds.id.get_create_plist()
+            prop_dcid = self.dataset.id.get_create_plist()
             if prop_dcid.get_nfilters() == 1 and prop_dcid.get_filter(0)[0] == 32001:
                 chunk = prop_dcid.get_chunk()
-                shape = ds.shape
+                shape = self.dataset.shape
                 frame_rank = len(shape) - scan_rank
                 if shape[-frame_rank:] == chunk[-frame_rank:] and all(
                     [i == 1 for i in chunk[:scan_rank]]
@@ -313,19 +278,18 @@ class FrameReader:
         >>>         frame_list.append(frame)
         """
 
-        ds = self.ds if self.ds is not None else self.h5file[self.dataset]
+        ds = self.dataset
         shape = ds.shape
 
-        if force_refresh and hasattr(ds, "refresh"):
-            ds.refresh()
+        if force_refresh:
+            refresh_dataset(ds)
 
         try:
             # might fail if dataset is cached
             pos = self.get_pos(index, shape)
         except ValueError:
             # refresh dataset and try again
-            if hasattr(ds, "refresh"):
-                ds.refresh()
+            refresh_dataset(ds)
 
             shape = ds.shape
             pos = self.get_pos(index, shape)
@@ -340,9 +304,9 @@ class FrameReader:
         if self.use_direct_chunk:
             return self.get_frame_direct(ds, pos, rank, slices)
         else:
-            return self.get_frame(ds, pos, rank, slices)
+            return self.get_frame(ds, slices)
 
-    def get_frame(self, ds, pos, rank, slices):
+    def get_frame(self, ds, slices):
         frame = ds[tuple(slices)]
         return frame, tuple(slices[: self.scan_rank])
 
@@ -356,8 +320,7 @@ class FrameReader:
         except Exception:
             # let the file system catch up
             sleep(1)
-            if hasattr(ds, "refresh"):
-                ds.refresh()
+            refresh_dataset(ds)
             out = ds.id.read_direct_chunk(chunk_pos)
 
         decom = blosc.decompress(out[1])
